"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.saveMotionInfo = void 0;
const package_name_1 = require("./package-name");
const gql_1 = require("@deep-foundation/deeplinks/imports/gql");
function saveMotionInfo(params) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const info = yield fixMotionInfo({ info: params.info });
        console.log({ params });
        const { deep } = params;
        const serialOperations = [];
        let deviceLink = yield getDeviceLink();
        let motionLinkId;
        let value;
        const motionLink = yield getMotionLinkOrUndefined({
            deviceLinkId: deviceLink.id,
        });
        if (motionLink) {
            motionLinkId = motionLink.id;
            value = (_a = motionLink.value) === null || _a === void 0 ? void 0 : _a.value;
        }
        else {
            const reservedLinkIds = yield deep.reserve(1);
            motionLinkId = reservedLinkIds.pop();
            serialOperations.push(yield getMotionLinkInsertSerialOperation({
                deviceLinkId: deviceLink.id,
                motionLinkId,
            }));
        }
        if (value) {
            serialOperations.push(yield getMotionLinkValueUpdateSerialOperation({ motionLinkId }));
        }
        else {
            serialOperations.push(yield getMotionLinkValueInsertSerialOperation({ motionLinkId }));
        }
        console.log({ serialOperations });
        yield deep.serial({
            operations: serialOperations,
        });
        function getDeviceLink() {
            return __awaiter(this, void 0, void 0, function* () {
                let deviceLink;
                if ('deviceLinkId' in params) {
                    const { data } = yield deep.select({
                        id: params.deviceLinkId,
                    });
                    deviceLink = data[0];
                }
                else if ('deviceLink' in params) {
                    deviceLink = params.deviceLink;
                }
                else {
                    throw new Error(`Either deviceLink or deviceLinkId must be passed`);
                }
                return deviceLink;
            });
        }
        function getMotionLinkOrUndefined({ deviceLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                const selectData = {
                    type_id: {
                        _id: [package_name_1.MOTION_PACKAGE_NAME, 'Motion'],
                    },
                    from_id: deviceLinkId,
                    to_id: deviceLinkId,
                };
                const { data: [motionLink], } = yield deep.select(selectData);
                // if (!motionLink) {
                //   throw new Error(`Select with data ${selectData} return empty result`)
                // }
                return motionLink;
            });
        }
        function getMotionLinkInsertSerialOperation({ deviceLinkId, motionLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return (0, gql_1.createSerialOperation)({
                    table: 'links',
                    type: 'insert',
                    objects: {
                        id: motionLinkId,
                        type_id: yield deep.id(package_name_1.MOTION_PACKAGE_NAME, 'Motion'),
                        from_id: deviceLinkId,
                        to_id: deviceLinkId,
                        in: {
                            data: {
                                type_id: yield deep.id('@deep-foundation/core', 'Contain'),
                                from_id: deviceLinkId,
                                string: {
                                    data: {
                                        value: 'Motion',
                                    },
                                },
                            },
                        },
                    },
                });
            });
        }
        function getMotionLinkValueInsertSerialOperation({ motionLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return (0, gql_1.createSerialOperation)({
                    table: 'objects',
                    type: 'insert',
                    objects: {
                        link_id: motionLinkId,
                        value: info,
                    },
                });
            });
        }
        function getMotionLinkValueUpdateSerialOperation({ motionLinkId, }) {
            return __awaiter(this, void 0, void 0, function* () {
                return (0, gql_1.createSerialOperation)({
                    table: 'objects',
                    type: 'update',
                    exp: {
                        link_id: motionLinkId,
                    },
                    value: {
                        value: Object.assign({}, info),
                    },
                });
            });
        }
        function fixMotionInfo({ info }) {
            return __awaiter(this, void 0, void 0, function* () {
                // capacitor-motion actually actually pass to us object with extra fields that are not specified in their typescript interface
                // @ts-ignore
                return Object.assign(Object.assign(Object.assign(Object.assign({}, ('acceleration' in info
                    ? {
                        acceleration: {
                            x: info.acceleration.x,
                            y: info.acceleration.y,
                            z: info.acceleration.z,
                        },
                    }
                    : {})), ('accelerationIncludingGravity' in info
                    ? {
                        accelerationIncludingGravity: {
                            x: info.accelerationIncludingGravity.x,
                            y: info.accelerationIncludingGravity.y,
                            z: info.accelerationIncludingGravity.z,
                        },
                    }
                    : {})), ('rotationRate' in info
                    ? {
                        rotationRate: {
                            alpha: info.rotationRate.alpha,
                            beta: info.rotationRate.beta,
                            gamma: info.rotationRate.gamma,
                        },
                    }
                    : {})), ('interval' in info ? { interval: info.interval } : {}));
            });
        }
    });
}
exports.saveMotionInfo = saveMotionInfo;
